-- script.on_configuration_changed(function()
script.on_event("upgrade-inserters", function(event)
  local RANGE = 10000
  local changecount = 0
  -- inserters = game.surfaces[1].find_entities_filtered{area = {{0 - RANGE, 0 - RANGE}, {RANGE, RANGE}}, type= "inserter"}
  inserters = game.surfaces[1].find_entities_filtered{
    area ={
      {game.players[event.player_index].position.x - RANGE, game.players[event.player_index].position.y - RANGE},
      {game.players[event.player_index].position.x + RANGE, game.players[event.player_index].position.y + RANGE}
    },
    type= "inserter"
  }
  for _,ins in pairs(inserters) do
    if string.match(ins.name, "left") or string.match(ins.name, "right") or string.match(ins.name, "near") then
      ReplaceInserter(ins)
      changecount = changecount + 1
    end
  end
  if changecount > 0 then
    game.players[event.player_index].print(changecount .. " side inserters upgraded.")
  else
    game.players[event.player_index].print("No side inserters found.")
  end
end)

function ReplaceInserter(ins)
  local itype = "inserter"
  local smart = false
  local filters = {}
  local fuel
  if string.sub(ins.name, 1, 6) == "burner" then
    if ins.get_inventory(defines.inventory.fuel).get_item_count() > 0 then
      fuel = ins.get_inventory(defines.inventory.fuel)[1]
    end
    itype = "burner-inserter"
  elseif string.sub(ins.name, 1, 4) == "fast" then
    itype = "fast-inserter"
  elseif string.sub(ins.name, 1, 4) == "long" then
    itype = "long-handed-inserter"
  elseif string.sub(ins.name, 1, 5) == "smart" then
    smart = true
    itype = "filter-inserter"
    for i = 1,5 do
      filters[i] = {index = i, name = ins.get_filter(i)}
    end
  end
  local position = ins.position
  local direction = ins.direction
  local left, right
  if string.match(ins.name, "left") then
    direction = (direction + 6) % 8
    left = true
  elseif string.match(ins.name, "right") then
    direction = (direction + 2) % 8
    right = true
  end
  local near = string.match(ins.name, "near") ~= nil
  local force = ins.force
  local energy = ins.energy
  local newins = game.surfaces[1].create_entity{
    name = itype,
    position = position,
    direction = direction,
    force = ins.force
  }
  newins.energy = energy
  if fuel then
    newins.insert(fuel)
  end
  if smart then
    for i = 1,5 do
      newins.set_filter(i, filters[i].name)
    end
  end
  ins.destroy()
  if left then
    RotateLeft(newins)
  elseif right then
    RotateRight(newins)
  end
  if near then
    ToggleInserterDrop(newins)
  end
end

function RotateLeft(entity)
  local px = entity.pickup_position.x - entity.position.x
  local py = entity.pickup_position.y - entity.position.y
  -- NOTE: inserter drop location is opposite to orientation direction
  if entity.direction == defines.direction.south then -- is north, go east
    entity.pickup_position = {entity.position.x - py, entity.position.y}
  elseif  entity.direction == defines.direction.west then -- is east, go south
    entity.pickup_position = {entity.position.x, entity.position.y + px}
  elseif entity.direction == defines.direction.north then -- is south, go west
    entity.pickup_position = {entity.position.x - py, entity.position.y}
  elseif entity.direction == defines.direction.east then -- is west, go north
    entity.pickup_position = {entity.position.x, entity.position.y + px}
  end
  -- set direction to force update of insert location
  entity.direction = entity.direction
end

function RotateRight(entity)
  local px = entity.pickup_position.x - entity.position.x
  local py = entity.pickup_position.y - entity.position.y
  -- NOTE: inserter drop location is opposite to orientation direction
  if entity.direction == defines.direction.south then -- is north, go west
    entity.pickup_position = {entity.position.x + py, entity.position.y}
  elseif  entity.direction == defines.direction.west then -- is east, go north
    entity.pickup_position = {entity.position.x, entity.position.y - px}
  elseif entity.direction == defines.direction.north then -- is south, go east
    entity.pickup_position = {entity.position.x + py, entity.position.y}
  elseif entity.direction == defines.direction.east then -- is west, go south
    entity.pickup_position = {entity.position.x, entity.position.y - px}
  end
  -- set direction to force update of insert location
  entity.direction = entity.direction
end

function ToggleInserterDrop(entity)
  local OFFSET = 0.3125
  local dx = entity.drop_position.x - entity.position.x
  local dy = entity.drop_position.y - entity.position.y
  if entity.direction == defines.direction.south or entity.direction == defines.direction.north then
    if dy % 1 > 0.5 then
      dy = dy + OFFSET
    else
      dy = dy - OFFSET
    end
  elseif entity.direction == defines.direction.west or entity.direction == defines.direction.east then
    if dx % 1 > 0.5 then
      dx = dx + OFFSET
    else
      dx = dx - OFFSET
    end
  end
  entity.drop_position = {entity.position.x + dx, entity.position.y + dy}
end